/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.seasar.framework.beans.BeanDesc;
import org.seasar.framework.beans.factory.BeanDescFactory;
import org.seasar.framework.container.ComponentDef;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.servlet.S2ContainerServlet;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.commons.util.ArrayUtil;
import org.seasar.tugboat.gtd.container.annotation.RestMethod;
import org.seasar.tugboat.gtd.container.constant.HttpHeader;
import org.seasar.tugboat.gtd.container.constant.HttpMethod;
import org.seasar.tugboat.gtd.container.constant.HttpStatus;
import org.seasar.tugboat.gtd.container.exception.InvalidSecureTokenRuntimeException;
import org.seasar.tugboat.gtd.container.exception.UnauthorizedRuntimeException;
import org.seasar.tugboat.gtd.webservices.service.DefaultRESTfulWebService;
import org.seasar.tugboat.gtd.webservices.service.impl.WebServiceAuthorizeImpl;

/**
 * RESTful webservice servlet.
 * 
 * @author yosuke
 * 
 */
public class RESTfulWebServiceServlet extends HttpServlet {
    private static final long serialVersionUID = -5053122847284990187L;

    private static final Logger LOG = Logger.getLogger(RESTfulWebServiceServlet.class);

    S2Container container;

    static final long AUTH_PASS = -1;

    static final int AUTH_NOT_ACCEPTABLE = -2;

    static final int AUTH_UNAUTHORIZED = -3;

    static final String PREFIX = "webservice.";

    static final String AUTH_CLASS_NAME = "authorize";

    @Override
    public void init(ServletConfig config) throws ServletException {
        this.container = S2ContainerServlet.getContainer();
    }

    @Override
    public void service(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        this.invokeMethod(request, response);
    }

    /**
     * invoke method.
     * 
     * @param request
     * @param response
     * @throws ServletException
     * @throws IOException
     */
    protected void invokeMethod(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        //
        // get parameters.
        //
        String httpMethodOverride = request.getHeader(HttpHeader.X_HTTP_METHOD_OVERRIDE);
        LOG.debug("httpMetofOverride:" + httpMethodOverride);

        String httpMethod = 
            httpMethodOverride == null || "".equals(httpMethodOverride) 
            ? request.getMethod() 
            : httpMethodOverride;

        String accept = request.getHeader(HttpHeader.ACCEPT);
        String wsse = request.getHeader(HttpHeader.X_WSSE);
        LOG.debug("http-method:" + httpMethod);
        LOG.debug("wsse:" + wsse);

        String[] pathInfo = StringUtils.split(request.getPathInfo(), '/');
        if (pathInfo == null || pathInfo.length == 0) {
            this.onFailure(response, "Invalid access error.", HttpStatus.NOT_ACCEPTABLE);
            return;
        }
        LOG.debug(request.getPathInfo());

        String componentName = PREFIX + pathInfo[0];
        String method = "";
        Map<String, Object> parameters = new HashMap<String, Object>();
        if (pathInfo.length == 1) {
        } else if (pathInfo.length == 2) {
            parameters.put("id", new String[] { pathInfo[1] });
        } else {
            String key = "";
            for (int i = 0; i < pathInfo.length; i++) {
                if (i == 0)
                    continue;
                if (i % 2 > 0)
                    key = pathInfo[i];
                if (i % 2 == 0 && "action".equals(key)) {
                    method = pathInfo[i];
                } else if (pathInfo[i].indexOf(",") > -1) {
                    parameters.put(key, StringUtils.split(pathInfo[i], ','));
                } else {
                    parameters.put(key, new String[] { pathInfo[i] });
                }
            }
        }
        if (httpMethod.equals(HttpMethod.POST.name()) || httpMethod.equals(HttpMethod.PUT.name())) {
            Enumeration<?> paramNames = request.getParameterNames();
            LOG.debug("paramNames:" + paramNames);
            
            while (paramNames.hasMoreElements()) {
                String key = (String) paramNames.nextElement();
                parameters.put(key, new String[] { request.getParameter(key) });

                LOG.debug("key:" + key);
            }
        }

        //
        // authorize.
        //
        long userId = AUTH_PASS;
        try {
            userId = this.authorize(request, response, wsse);
        } catch (UnauthorizedRuntimeException e) {
            this.onFailure(response, e.getMessage(), HttpStatus.UNAUTHORIZED);
            return;
        } catch (InvalidSecureTokenRuntimeException e) {
            this.onFailure(response, e.getMessage(), HttpStatus.NOT_ACCEPTABLE);
            return;
        }

        //
        // invoke method.
        //
        ComponentDef component = this.getComponentDefNoException(componentName);
        Object obj = component.getComponent();

        if ("".equals(method)) {
            method = this.getMethodName(obj, httpMethod, parameters);

            LOG.debug("method:[" + method + "]");
            LOG.debug("---");
        }
        if ("".equals(method)) {
            this.onFailure(response, "Not Found Exception.", HttpStatus.NOT_FOUND);
            return;
        }

        Object target = null;
        try {
            BeanDesc beanDesc = BeanDescFactory.getBeanDesc(component.getConcreteClass());
            beanDesc.invoke(obj, DefaultRESTfulWebService.SETTER_ACCEPT_TYPE,
                    new Object[] { accept });
            beanDesc.invoke(obj, DefaultRESTfulWebService.SETTER_USER_ID, new Object[] { userId });
            this.setWriteProperties(beanDesc, obj, parameters, userId);

            target = beanDesc.invoke(obj, method, null);
            this.onSuccess(response, target, accept);

        } catch (UnauthorizedRuntimeException e) {
            this.onFailure(response, e.getMessage(), HttpStatus.UNAUTHORIZED);

        } catch (Exception e) {
            e.printStackTrace();
            this.onFailure(response, "The error occurred while create Rest Web Service response."
                    + e.getMessage(), HttpStatus.INTERNAL_ERROR);
        }
    }

    /**
     * set write properties.
     * 
     * @param beanDesc
     * @param obj
     * @param parameters
     * @param userId
     * @throws Exception
     */
    protected void setWriteProperties(BeanDesc beanDesc, Object obj,
            Map<String, Object> parameters, long userId) throws Exception {

        Iterator<?> p = parameters.keySet().iterator();
        Method[] methods = obj.getClass().getMethods();

        while (p.hasNext()) {
            String key = (String) p.next();

            for (Method m : methods) {
                if (m.getName().indexOf("set") > -1
                        && key.toLowerCase().equals(
                                StringUtils.replace(m.getName().toLowerCase(), "set", ""))) {
                    beanDesc.invoke(obj, m.getName(), this.generateParameterValues(m
                            .getParameterTypes()[0], (Object[]) parameters.get(key)));
                    break;
                }
            }
        }
    }

    /**
     * authorize
     * 
     * @param request
     * @param response
     * @param wsse
     * @return - userid
     * @throws Exception
     */
    protected long authorize(HttpServletRequest request, HttpServletResponse response, String wsse)
            throws UnauthorizedRuntimeException {

        long id = AUTH_PASS;
        if (request.getPathInfo().indexOf(AUTH_CLASS_NAME) > -1)
            return id;

        WebServiceAuthorizeImpl authService = (WebServiceAuthorizeImpl) this
                .getComponentDefNoException(PREFIX + AUTH_CLASS_NAME).getComponent();
        authService.setWsse(wsse);

        return authService.authorize();
    }

    /**
     * get method name.
     * 
     * @param component
     * @param httpMethod
     * @param parameters
     * @return
     */
    protected String getMethodName(Object component, String httpMethod, Map<?, ?> parameters) {
        String method = "";
        Method[] methods = component.getClass().getMethods();

        for (Method m : methods) {
            RestMethod annotation = m.getAnnotation(RestMethod.class);

            if (annotation == null)
                continue;
            HttpMethod[] httpMethods = annotation.methods();
            String[] requestParameters = annotation.parameters();

            for (HttpMethod hm : httpMethods) {
                if (annotation != null && hm.name().equals(httpMethod)) {
                    if (parameters.size() == 0 && "".equals(requestParameters[0])) {
                        method = m.getName();
                        break;

                    } else if (parameters.size() == requestParameters.length) {
                        boolean isFound = false;

                        for (String p : requestParameters) {
                            isFound = false;
                            Iterator<?> it = parameters.keySet().iterator();
                            while (it.hasNext()) {
                                if (p.equals((String) it.next())) {
                                    isFound = true;
                                    break;
                                }
                            }
                            if (!isFound)
                                break;
                        }
                        if (isFound) {
                            method = m.getName();
                            break;
                        }
                    } else {
                        // error...
                    }
                }
            }
        }
        return method;
    }

    /**
     * generate parameter values.
     * 
     * @param parameterType
     * @param values
     * @return
     */
    protected Object[] generateParameterValues(Class<?> parameterType, Object[] values)
            throws Exception {
        Object[] result = null;
        if (!parameterType.isArray()) {
            if (parameterType.equals(int.class))
                result = new Object[] { new Integer((String) values[0]).intValue() };
            else if (parameterType.equals(long.class))
                result = new Object[] { new Long((String) values[0]).longValue() };
            else if (parameterType.equals(short.class))
                result = new Object[] { new Short((String) values[0]).shortValue() };
            else if (parameterType.equals(float.class))
                result = new Object[] { new Float((String) values[0]).floatValue() };
            else if (parameterType.equals(double.class))
                result = new Object[] { new Double((String) values[0]).doubleValue() };
            else if (parameterType.equals(boolean.class))
                result = new Object[] { new Boolean((String) values[0]).booleanValue() };
            else if (parameterType.equals(String.class))
                result = new Object[] { new String(((String) values[0]).getBytes("8859_1"), "UTF-8") };
            else if (parameterType.equals(Integer.class))
                result = new Object[] { new Integer((String) values[0]) };
            else if (parameterType.equals(Long.class))
                result = new Object[] { new Long((String) values[0]) };
            else if (parameterType.equals(Short.class))
                result = new Object[] { new Short((String) values[0]) };
            else if (parameterType.equals(Float.class))
                result = new Object[] { new Float((String) values[0]) };
            else if (parameterType.equals(Double.class))
                result = new Object[] { new Double((String) values[0]) };
            else if (parameterType.equals(Boolean.class))
                result = new Object[] { new Boolean((String) values[0]) };
            else
                result = new Object[] { values[0] };
        } else {
            List<String> list = Arrays.asList((String[]) values);
            if (parameterType.equals(int[].class))
                result = new Object[] { ArrayUtils.toPrimitive(ArrayUtil
                        .stringArrayToIntegerArray(list)) };
            else if (parameterType.equals(long[].class))
                result = new Object[] { ArrayUtils.toPrimitive(ArrayUtil
                        .stringArrayToLongArray(list)) };
            else if (parameterType.equals(short[].class))
                result = new Object[] { ArrayUtils.toPrimitive(ArrayUtil
                        .stringArrayToShortArray(list)) };
            else if (parameterType.equals(float[].class))
                result = new Object[] { ArrayUtils.toPrimitive(ArrayUtil
                        .stringArrayToFloatArray(list)) };
            else if (parameterType.equals(double[].class))
                result = new Object[] { ArrayUtils.toPrimitive(ArrayUtil
                        .stringArrayToDoubleArray(list)) };
            else if (parameterType.equals(boolean[].class))
                result = new Object[] { ArrayUtils.toPrimitive(ArrayUtil
                        .stringArrayToBooleanArray(list)) };
            else if (parameterType.equals(String[].class))
                result = new Object[] { list.toArray(new String[list.size()]) };
            else if (parameterType.equals(Integer[].class))
                result = new Object[] { ArrayUtil.stringArrayToIntegerArray(list) };
            else if (parameterType.equals(Long[].class))
                result = new Object[] { ArrayUtil.stringArrayToLongArray(list) };
            else if (parameterType.equals(Short[].class))
                result = new Object[] { ArrayUtil.stringArrayToShortArray(list) };
            else if (parameterType.equals(Float[].class))
                result = new Object[] { ArrayUtil.stringArrayToFloatArray(list) };
            else if (parameterType.equals(Double[].class))
                result = new Object[] { ArrayUtil.stringArrayToDoubleArray(list) };
            else if (parameterType.equals(Boolean[].class))
                result = new Object[] { ArrayUtil.stringArrayToBooleanArray(list) };
            else
                result = new Object[] { ArrayUtil.stringArrayToObjectArray(list) };
        }
        return result;
    }

    /**
     * get component def.
     * 
     * @param componentName
     * @return
     */
    protected ComponentDef getComponentDefNoException(String componentName) {
        if (!this.container.hasComponentDef(componentName)) {
            return null;
        }
        return this.container.getComponentDef(componentName);
    }

    /**
     * on success.
     * 
     * @param response
     * @param target
     * @throws IOException
     */
    protected void onSuccess(HttpServletResponse response, Object target, String accept)
            throws IOException {
        response.setHeader("Cache-Control", "no-cache");
        response.setHeader("Pragma", "no-cache");
        response.setContentType(accept + "; charset=utf-8");
        response.setCharacterEncoding("UTF-8");

        PrintWriter pw = response.getWriter();
        pw.write(String.valueOf(target));
        pw.close();
    }

    /**
     * on failure.
     * 
     * @param response
     * @param message
     * @param statusCode
     * @throws IOException
     */
    protected void onFailure(HttpServletResponse response, String message, int statusCode)
            throws IOException {

        response.setHeader("Cache-Control", "no-cache");
        response.setHeader("Pragma", "no-cache");
        response.setStatus(statusCode);

        PrintWriter pw = response.getWriter();
        pw.write(message);
        pw.close();
    }
}
