/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.helper.impl;

import java.lang.reflect.InvocationTargetException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.beanutils.BeanUtils;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskToTaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskViewStateRelationDto;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroupRelation;
import org.seasar.tugboat.gtd.contents.task.helper.TaskHelper;

/**
 * task helper impl.
 * 
 * @author yosuke
 * 
 */
public class TaskHelperImpl implements TaskHelper {

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createActivitiesJsonForTask(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto[])
     */
    public String createActivitiesJsonForTask(ActivityDto[] activities) {
        if (activities == null || activities.length == 0)
            return null;

        List<Map<?, ?>> listJson = new ArrayList<Map<?, ?>>();
        for (ActivityDto c : activities) {
            Map<String, String> map = new HashMap<String, String>();
            map.put("id", String.valueOf(c.getId()));
            map.put("value", c.getName());
            map.put("color", c.getColorValue());
            map.put("token", "/rss/" + c.getFeedToken() + "/tasks.rss");
            listJson.add(map);
        }
        JSONArray jsonArray =
            JSONArray.fromObject(
                listJson.toArray(new Map[listJson.size()]));
        return jsonArray.toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskListJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationsByActivityDto[],
     *      int, int, int, long)
     */
    public String createTasksJSON(TaskRelationDto[] tasks, int totalRecords,
            int currentPageNumber, int recordPerPage, long userId) {

        if (tasks == null || tasks.length == 0)
            return "";

        List<Map<?, ?>> listJson = new ArrayList<Map<?, ?>>();
        for (TaskRelationDto t : tasks) {
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("id", t.getTaskId());
            map.put("activityId", t.getActivityId());
            map.put("activityName", t.getActivityValue());
            map.put("activityColor", t.getColorValue());
            map.put("title", t.getTitle());
            map.put("statusId", t.getStatusId());
            map.put("classId", t.getClassificationId());
            map.put("postedBy", t.getUserLastName() + ","+ t.getUserFirstName());
            map.put("postedAt", t.getRegisteredFormated());
            map.put("personInChargeLastName", t.getPersonInChargeLastName());
            map.put("personInChargeFirstName", t.getPersonInChargeFirstName());
            map.put("due", t.getDeadlineFormated());
            map.put("isMe", t.getPersonInChargeId() == userId);
            map.put("taskGroupId", t.getTaskGroupId());
            map.put("taskGroupName", t.getTaskGroupName() == null ? "" : t.getTaskGroupName());

            if (t.getTaskTagDtos().length > 0) {
                List<Map<?, ?>> tagList = new ArrayList<Map<?, ?>>();
                for (TaskTagDto tag : t.getTaskTagDtos()) {
                    Map<String, Object> jsonTag = new HashMap<String, Object>();
                    jsonTag.put("tagId", tag.getTagId());
                    jsonTag.put("tagValue", tag.getTagValue());
                    tagList.add(jsonTag);
                }
                JSONArray jsonTagArray = 
                    JSONArray.fromObject(tagList.toArray(new Map[tagList.size()]));
                map.put("tags", jsonTagArray);
            }
            listJson.add(map);
        }
        JSONArray jsonArray = 
            JSONArray.fromObject(
                listJson.toArray(new Map[listJson.size()]));

        Map<String, Integer> page = new HashMap<String, Integer>();
        page.put("results", new Integer(totalRecords));
        page.put("totalPages", new Integer(totalRecords / recordPerPage
                + (totalRecords % recordPerPage == 0 ? 0 : 1)));

        if (currentPageNumber < 1) currentPageNumber = 1;
        
        page.put("currentPageNumber", new Integer(currentPageNumber));

        JSONObject json = JSONObject.fromObject(page);
        StringBuilder buf = new StringBuilder();
        buf.append("({tasks:")
           .append(jsonArray.toString())
           .append(",page:")
           .append(json.toString())
           .append("})");
        
        return buf.toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createDefaultFolderTasksJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskDto)
     */
    public String createDefaultFolderTasksJSON(TaskRelationDto folderTasks) {
        if (folderTasks == null)
            return null;

        Map<String, String> map = new HashMap<String, String>();
        map.put("inboxTasks", String.valueOf(folderTasks.getInboxTasks()));
        map.put("todayTasks", String.valueOf(folderTasks.getTodayTasks()));
        map.put("weekTasks", String.valueOf(folderTasks.getWeekTasks()));
        map.put("monthTasks", String.valueOf(folderTasks.getMonthTasks()));
        map.put("holdTasks", String.valueOf(folderTasks.getHoldTasks()));
        map.put("somedayTasks", String.valueOf(folderTasks.getSomedayTasks()));
        map.put("todayToDoTasks", String.valueOf(folderTasks.getTodayTodoTasks()));
        map.put("todayDoingTasks", String.valueOf(folderTasks.getTodayDoingTasks()));
        map.put("todayDoneTasks", String.valueOf(folderTasks.getTodayDoneTasks()));
        map.put("archiveTasks", String.valueOf(folderTasks.getArchiveTasks()));
        map.put("trashBoxTasks", String.valueOf(folderTasks.getTrashBoxTasks()));
        map.put("allTasks", String.valueOf(folderTasks.getTodayTasks()
                + folderTasks.getWeekTasks() + folderTasks.getMonthTasks()
                + folderTasks.getHoldTasks() + folderTasks.getSomedayTasks()));

        JSONObject json = JSONObject.fromObject(map);
        return json.toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskCommentsJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskCommentDto[])
     */
    @SuppressWarnings("unchecked")
    public String createTaskCommentsJSON(long userId, TaskCommentDto[] comments) {
        if (comments == null || comments.length == 0)
            return null;
        
        try {
            List<Map<?, ?>> listJson = new ArrayList<Map<?, ?>>();
            
            for (TaskCommentDto comment : comments) {
                comment.setIsMyComment(userId == comment.getUserId());
                Map<String, Object> map = BeanUtils.describe(comment);
                map.put("isMyComment", comment.getIsMyComment());
                listJson.add(map);
            }
            JSONArray jsonArray = 
                JSONArray.fromObject(
                    listJson.toArray(new Map[listJson.size()]));

            return jsonArray.toString();
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskDetailJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto[])
     */
    public String createTaskDetailJSON(TaskRelationDto taskDto, TaskTagDto[] tagDto) {
        if (taskDto == null)
            return null;
        
        try {
            Map<?, ?> map = BeanUtils.describe(taskDto);
            JSONObject json = JSONObject.fromObject(map);

            List<Map<?, ?>> listJson = new ArrayList<Map<?, ?>>();
            if (tagDto != null && tagDto.length > 0) {
                for (TaskTagDto t : tagDto) {
                    listJson.add(BeanUtils.describe(t));
                }
            }
            JSONArray jsonArray = 
                JSONArray.fromObject(
                    listJson.toArray(new Map[listJson.size()]));

            StringBuilder buf = new StringBuilder();
            buf.append("{task:")
               .append(json.toString())
               .append(",tags:")
               .append(jsonArray.toString())
               .append("}");
            return buf.toString();

        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskForBalloonJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskDto)
     */
    public String createBalloonDetailJSON(TaskRelationDto taskDto) {
        if (taskDto == null)
            return null;

        Map<String, Object> map = new HashMap<String, Object>();
        map.put("id", taskDto.getTaskId());
        map.put("title", taskDto.getTitle());
        map.put("activityId", taskDto.getActivityId());
        map.put("activityColor", taskDto.getColorValue());
        map.put("activityName", taskDto.getActivityValue());
        map.put("personName", taskDto.getPersonInChargeLastName() + "," + taskDto.getPersonInChargeFirstName());
        map.put("personId", taskDto.getPersonInChargeId());
        map.put("dueFormated", taskDto.getDeadlineFormated());
        map.put("dueValue", taskDto.getDeadline());
        map.put("postedBy", taskDto.getUserLastName() + "," + taskDto.getUserFirstName());
        map.put("postedAt", taskDto.getRegisteredFormated());
        map.put("statusId", taskDto.getStatusId());
        map.put("classId", taskDto.getClassificationId());
        map.put("description", taskDto.getDescription());
        map.put("taskGroupId",taskDto.getTaskGroupId());
        map.put("taskGroupName",taskDto.getTaskGroupName() == null ? "" : taskDto.getTaskGroupName());

        // set task tags.
        try {
            JSONObject json = JSONObject.fromObject(map);
            TaskTagDto[] taskTags = taskDto.getTaskTagDtos();
            List<Map<?, ?>> listJson = new ArrayList<Map<?, ?>>();
            
            for (TaskTagDto t : taskTags) {
                Map<?, ?> mapTaskTag = BeanUtils.describe(t);
                listJson.add(mapTaskTag);
            }
            JSONArray jsonArray = JSONArray.fromObject(listJson
                    .toArray(new Map[listJson.size()]));

            StringBuilder buf = new StringBuilder();
            buf.append("{task:")
               .append(json.toString())
               .append(",tags:")
               .append(jsonArray.toString())
               .append("}");
            return buf.toString();
            
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (NoSuchMethodException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskToTasksAndTrackbacksJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskToTaskDto[],
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTasksDto[])
     */
    public String createTaskToTasksJSON(
            TaskToTaskDto[] taskToTaskDtos, MeetingToTaskDto[] meetingToTaskDtos) {

        if ((taskToTaskDtos == null || taskToTaskDtos.length == 0)
                && (meetingToTaskDtos == null || meetingToTaskDtos.length == 0))
            return null;

        List<Map<?, ?>> taskToTasksJson = new ArrayList<Map<?, ?>>();
        List<Map<?, ?>> meetingToTasksJson = new ArrayList<Map<?, ?>>();

        try {
            if (taskToTaskDtos != null && taskToTaskDtos.length > 0) {
                for (TaskToTaskDto dto : taskToTaskDtos) {
                    Map<?, ?> map = BeanUtils.describe(dto);
                    taskToTasksJson.add(map);
                }
            }
            if (meetingToTaskDtos != null && meetingToTaskDtos.length > 0) {
                for (MeetingToTaskDto dto : meetingToTaskDtos) {
                    Map<?, ?> map = BeanUtils.describe(dto);
                    meetingToTasksJson.add(map);
                }
            }
            JSONArray jsonArrayTaskToTasks = 
                JSONArray.fromObject(
                    taskToTasksJson.toArray(
                        new Map[taskToTasksJson.size()]));

            JSONArray jsonArrayMeetingToTasks = JSONArray
                    .fromObject(meetingToTasksJson
                            .toArray(new Map[meetingToTasksJson.size()]));

            StringBuilder buf = new StringBuilder();
            buf.append("({taskToTasks:")
               .append(jsonArrayTaskToTasks.toString())
               .append(", meetingToTasks:")
               .append(jsonArrayMeetingToTasks.toString())
               .append("})");
            return buf.toString();

        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (NoSuchMethodException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createBeforeViewStatusesJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskViewStateRelationDto)
     */
    public String createBeforeViewStatusesJSON(
            TaskViewStateRelationDto viewStatuses, long userId) {
        StringBuilder bufViewStatuses = new StringBuilder();
        bufViewStatuses.append("{");

        if (viewStatuses.getTaskSelectViewDto().getIsVisibleBrowser() == 0) {
            bufViewStatuses.append("statuses:[null],")
                    .append("classes:[null],").append("activities:[null],")
                    .append("people:[" + userId + "],");
        } else {
            JSONArray jsonStatuses = JSONArray.fromObject(viewStatuses.getTaskBrowseStatuses());
            bufViewStatuses.append("statuses:").append(jsonStatuses.toString()).append(",");

            JSONArray jsonClasses = JSONArray.fromObject(viewStatuses.getTaskBrowseClassifications());
            bufViewStatuses.append("classes:").append(jsonClasses.toString()).append(",");

            JSONArray jsonActivity = JSONArray.fromObject(viewStatuses.getTaskBrowseActivities());
            bufViewStatuses.append("activities:").append(jsonActivity.toString()).append(",");

            JSONArray jsonPerson = JSONArray.fromObject(viewStatuses.getTaskBrowsePersons());
            bufViewStatuses.append("people:").append(jsonPerson.toString()).append(",");
        }
        bufViewStatuses.append("select_class:")
                       .append(viewStatuses.getTaskSelectViewDto().getClassId())
                       .append(",")
                       .append("select_mode:")
                       .append(viewStatuses.getTaskSelectViewDto().getModeId())
                       .append(",")
                       .append("select_tab:")
                       .append(viewStatuses.getTaskSelectViewDto().getSelectedTabId())
                       .append(",")
                       .append("is_visible_browser:")
                       .append(viewStatuses.getTaskSelectViewDto().getIsVisibleBrowser())
                       .append("}");
        return bufViewStatuses.toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTagsListHTML(
     *      org.seasar.tugboat.gtd.contents.task.dto.TagDto[])
     */
    public String createTagsHTML(TagDto[] tagDto) {
        StringBuilder buf = new StringBuilder();
        buf.append("<ul class=\"list_for_auto_completer\" ")
           .append("xmlns:tgb=\"http://tugboat.seasar.org/ns/compatibility/annotations/1.0\" ")
           .append(">");

        if (tagDto == null || tagDto.length == 0) {
        } else {
            for (TagDto t : tagDto) {
                buf.append("<li ")
                   .append("tgb:value=\"")
                   .append(t.getId())
                   .append("\">")
                   .append(t.getValue())
                   .append("</li>");
            }
        }
        buf.append("</ul>");
        return buf.toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskTagsJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto[])
     */
    public String createTaskTagsJSON(TaskTagDto[] taskTags) {
        if (taskTags == null) return null;

        try {
            List<Map<?, ?>> list = new ArrayList<Map<?, ?>>();
            
            for (TaskTagDto t : taskTags) {
                Map<?, ?> map = BeanUtils.describe(t);
                list.add(map);
            }
            JSONArray jsonArray =
                JSONArray.fromObject(
                    list.toArray(new Map[list.size()]));
            return jsonArray.toString();

        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (NoSuchMethodException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createOwnTagsJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto[])
     */
    public String createOwnTagsJSON(TaskTagDto[] taskTags) {
        return this.createTaskTagsJSON(taskTags);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createDueListJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto[])
     */
    @SuppressWarnings("unchecked")
    public String createCalendarDeadlinesJSON(TaskRelationDto[] taskRelations) {
        if (taskRelations == null || taskRelations.length == 0)
            return "";

        try {
            List<Map> list = new ArrayList<Map>();

            DateFormat dfShort = new SimpleDateFormat("yyyyMMdd");
            Calendar tempCal = new GregorianCalendar();
            int todayNum = Integer.parseInt(dfShort.format(tempCal.getTime()));

            tempCal.add(Calendar.DAY_OF_MONTH, 1);
            int tomorrowNum = Integer.parseInt(dfShort
                    .format(tempCal.getTime()));

            tempCal = new GregorianCalendar();
            tempCal.add(Calendar.DAY_OF_MONTH, 3);
            int afterThreeDaysNum = Integer.parseInt(dfShort.format(tempCal
                    .getTime()));

            // past due.
            int passDueTasks = 0;
            int passDueDones = 0;
            for (TaskRelationDto dto : taskRelations) {
                int deadlineInt = Integer.parseInt(dto.getDeadline());
                if (todayNum <= deadlineInt)
                    break;

                passDueTasks += dto.getTotalRows();
                passDueDones += dto.getDoneRows();
            }
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("due", "00000000"); // past-due's flag.
            map.put("due_formatted", "Past due");
            map.put("total_rows", passDueTasks);
            map.put("done_rows", passDueDones);
            map.put("color", "red");
            list.add(map);

            Calendar cal = null;
            for (TaskRelationDto dto : taskRelations) {
                int deadlineInt = Integer.parseInt(dto.getDeadline());
                if (todayNum > deadlineInt)
                    continue;

                int deadlineNum = Integer.parseInt(dto.getDeadline());
                String deadlineStr = 
                    deadlineNum == todayNum    ? "Today due"    : 
                    deadlineNum == tomorrowNum ? "Tomorrow due" :
                                                 dto.getDeadlineFormated();

                map = new HashMap<String, Object>();
                map.put("due", dto.getDeadline());
                map.put("due_formatted", deadlineStr);
                map.put("total_rows", dto.getTotalRows());
                map.put("done_rows", dto.getDoneRows());

                cal = new GregorianCalendar();
                cal.setTime(new SimpleDateFormat("yyyyMMdd").parse(dto
                        .getDeadline()));
                int dueNum = Integer.parseInt(dfShort.format(cal.getTime()));

                map.put(
                    "color",
                    todayNum >= dueNum          ? "red"   : 
                    afterThreeDaysNum >= dueNum ? "yellow":
                                                  "blue");
                list.add(map);
            }
            JSONArray jsonArray = 
                JSONArray.fromObject(
                    list.toArray(new Map[list.size()]));
            return jsonArray.toString();

        } catch (ParseException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskGroupListJSON(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskGroupRelation[])
     * 
     * [{"doneTasks":1,"activityId":9990,"activityName":"activity-0","allTasks":3,
     *   "taskGroups":[
     *     {"id",1,"name":"abc"},{"id",1,"name":"abc"}
     *   ]
     * }]
     */
    public String createTaskGroupsJSON(TaskGroupRelation[] activeGroups, TaskGroupRelation[] archiveGroups) {
        String activeGroupsJson = "null";
        if (activeGroups != null && activeGroups.length > 0) {
            Iterator<TaskGroupRelation> itActiveGroups = Arrays.asList(activeGroups).iterator();
            activeGroupsJson = this.genTaskGroupsJSON(itActiveGroups, activeGroups.length);
        }
        
        String archiveGroupsJson = "null";
        if (archiveGroups != null && archiveGroups.length > 0) {
            Iterator<TaskGroupRelation> itArchiveGroups = Arrays.asList(archiveGroups).iterator();
            archiveGroupsJson = this.genTaskGroupsJSON(itArchiveGroups, archiveGroups.length);
        }
        return ("({active:" + activeGroupsJson + ",archive:" + archiveGroupsJson + "})");
    }
    
    /**
     * generate task groups json.
     * 
     * @param itTaskGroups
     * @return
     */
    private String genTaskGroupsJSON(Iterator<TaskGroupRelation> itTaskGroups, int size) {
        long beforeActivityId = 0L;
        int counter = 0;
        
        List<Map<String, Object>> listParent = new ArrayList<Map<String, Object>>();
        List<Map<String, Object>> listChild = null;
        Map<String, Object> mapParent = null;

        do {
            TaskGroupRelation taskGroupRelation =
                itTaskGroups.hasNext() ? itTaskGroups.next() : null;

            if (taskGroupRelation == null) {
                mapParent.put("taskGroups", listChild);
                listParent.add(mapParent);
                break;
            } else {
                if (beforeActivityId == 0L) {
                    mapParent = this.getActivityMap(taskGroupRelation);
                    
                    listChild = new ArrayList<Map<String, Object>>();
                    listChild.add(this.getTaskGroupMap(taskGroupRelation));
                    
                } else if (beforeActivityId != taskGroupRelation.getActivityId()) { 
                    mapParent.put("taskGroups", listChild);
                    listParent.add(mapParent);
                    
                    mapParent = this.getActivityMap(taskGroupRelation);
                    listChild = new ArrayList<Map<String, Object>>();
                    listChild.add(this.getTaskGroupMap(taskGroupRelation));
                    
                } else {
                    listChild.add(this.getTaskGroupMap(taskGroupRelation));
                }
            }
            beforeActivityId = taskGroupRelation.getActivityId();
            counter++;
        } while (counter <= size);
        
        JSONArray jsonArray = 
            JSONArray.fromObject(
                listParent.toArray(new Map[listParent.size()]));
        return jsonArray.toString();
    }
    
    /**
     * get activity map.
     * 
     * @param taskGroupRelation
     * @return
     */
    private Map<String, Object> getActivityMap(TaskGroupRelation taskGroupRelation) {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("activityId", new Long(taskGroupRelation.getActivityId()));
        map.put("activityName", taskGroupRelation.getActivityName());
        map.put("activityColor", taskGroupRelation.getActivityColor());
        return map;
    }
    
    /**
     * get task group map.
     * 
     * @param taskGroupRelation
     * @return
     */
    private Map<String, Object> getTaskGroupMap(TaskGroupRelation taskGroupRelation) {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("id", new Long(taskGroupRelation.getTaskGroupId()));
        map.put("name", taskGroupRelation.getName());
        map.put("all", new Integer(taskGroupRelation.getAllTasks()));
        map.put("done", new Integer(taskGroupRelation.getDoneTasks()));
        map.put("activityId", new Long(taskGroupRelation.getActivityId()));
        return map;
    }
    
    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskGroupJSON(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskGroupRelation)
     */
    public String createTaskGroupJSON(TaskGroupRelation taskGroupRelation) {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("activityId", taskGroupRelation.getActivityId());
        map.put("taskGroupId", taskGroupRelation.getTaskGroupId());
        map.put("taskGroupName", taskGroupRelation.getName());
        map.put("startDate", taskGroupRelation.getStartDate());
        map.put("deadline",
            taskGroupRelation.getDeadline() == null ? "" : taskGroupRelation.getDeadline().trim());
        map.put("description", taskGroupRelation.getDescription());
        
        JSONObject json = JSONObject.fromObject(map);
        return json.toString();
    }
    
    /*
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createNewTaskIdsJSON(long[])
     */
    public String createNewTaskIdsJSON(long[] taskIds) {
        StringBuilder buf = new StringBuilder();
        buf.append("({task_ids:[");
        for (long id : taskIds) {
            buf.append(id).append(",");
        }
        buf.delete(buf.length()-1, buf.length());
        buf.append("]})");
        return buf.toString();
    }
}
